unit UComp;

interface

{
  Test program for the Unsigned unit.  Enter two values and
  press the appropriate button to see the result.  Click the
  Compare button to run CompareUnsigned.  Click the Test button
  to run random tests, comparing with a 386-based oracle.

  Copyright  1995 Tempest Software
}

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Num1: TEdit;
    Label2: TLabel;
    Num2: TEdit;
    LTButton: TButton;
    LEButton: TButton;
    NEButton: TButton;
    EQButton: TButton;
    GEButton: TButton;
    GTButton: TButton;
    Label3: TLabel;
    Result: TEdit;
    CompareButton: TButton;
    TestButton: TButton;
    UInt1: TEdit;
    UInt2: TEdit;
    UIntCompareButton: TButton;
    DivButton: TButton;
    ModButton: TButton;
    procedure LTButtonClick(Sender: TObject);
    procedure LEButtonClick(Sender: TObject);
    procedure EQButtonClick(Sender: TObject);
    procedure NEButtonClick(Sender: TObject);
    procedure GEButtonClick(Sender: TObject);
    procedure GTButtonClick(Sender: TObject);
    procedure CompareButtonClick(Sender: TObject);
    procedure TestButtonClick(Sender: TObject);
    procedure Num1Change(Sender: TObject);
    procedure Num2Change(Sender: TObject);
    procedure UIntCompareButtonClick(Sender: TObject);
    procedure DivButtonClick(Sender: TObject);
    procedure ModButtonClick(Sender: TObject);
  public
    function Value1: LongInt;
    function Value2: LongInt;
    procedure SetResult(Value: Boolean);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Unsigned;

function TForm1.Value1: LongInt;
begin
  if Num1.Text = '' then
    Num1.Text := '0';
  Result := StrToInt(Num1.Text);
end;

function TForm1.Value2: LongInt;
begin
  if Num2.Text = '' then
    Num2.Text := '0';
  Result := StrToInt(Num2.Text);
end;

procedure TForm1.SetResult(Value: Boolean);
begin
  case Value of
  True:  Result.Text := 'True';
  False: Result.Text := 'False';
  end;
end;

procedure TForm1.LTButtonClick(Sender: TObject);
begin
 SetResult(ULT(Value1, Value2));
end;

procedure TForm1.LEButtonClick(Sender: TObject);
begin
 SetResult(ULE(Value1, Value2));
end;

procedure TForm1.EQButtonClick(Sender: TObject);
begin
 SetResult(Value1 = Value2);
end;

procedure TForm1.NEButtonClick(Sender: TObject);
begin
 SetResult(Value1 <> Value2);
end;

procedure TForm1.GEButtonClick(Sender: TObject);
begin
 SetResult(UGE(Value1, Value2));
end;

procedure TForm1.GTButtonClick(Sender: TObject);
begin
 SetResult(UGT(Value1, Value2));
end;

procedure TForm1.CompareButtonClick(Sender: TObject);
begin
 Result.Text := IntToStr(CompareUnsigned(Value1, Value2));
end;

function CompareOracle(A, B: LongInt): Integer;
{$ifdef WIN32}
var
  AComp, BComp: Comp;
begin
  AComp := UIntToComp(A);
  BComp := UIntToComp(B);
  if AComp < BComp then
    Result := -1
  else if AComp > BComp then
    Result := 1
  else
    Result := 0;
end;
{$else}
assembler;
asm
  xor ax, ax
  db 66h
  mov bx, A.Word[0]
  db 66h
  mov cx, B.Word[0]
  db 66h
  sub bx, cx
  ja @@1           { A > B }
  je @@2           { A = B }
  dec ax           { A < B }
  jmp @@2
@@1:
  inc ax
@@2:
end;
{$endif}

procedure OneTest;
var
  Val1, Val2: LongInt;
  Cmp1, Cmp2: Integer;
begin
  LongRec(Val1).Lo := Random($FFFF);
  LongRec(Val2).Lo := Random($FFFF);
  Val1 := Val1 shl Random(16);
  Val2 := Val2 shl Random(16);
  LongRec(Val1).Lo := Random($FFFF);
  LongRec(Val2).Lo := Random($FFFF);
  Cmp1 := CompareUnsigned(Val1, Val2);
  Cmp2 := CompareOracle(Val1, Val2);
  if (Cmp1 < 0) and (Cmp2 < 0) then
    Exit
  else if (Cmp1 > 0) and (Cmp2 > 0) then
    Exit
  else if (Cmp1 = 0) and (Cmp2 = 0) then
    Exit
  else
    raise Exception.CreateFmt('Oops. Compare($%x, $%x)'#13#10'Asm=%d, Unsigned=%d', [Val1, Val2, Cmp2, Cmp1]);
end;

{ Remember Value between calls to the random tester. }
var
  Value: string;

procedure TForm1.TestButtonClick(Sender: TObject);
var
  I: LongInt;
begin
  if Test8086 < 2 then
    raise Exception.Create('Requires a 386 or better to run Tests.');
  if Value = '' then
    Value := '1000';
  if InputQuery('Random Test', 'Enter the number of tests to perform:', Value) then
  begin
    Screen.Cursor := crHourglass;
    Randomize;
    Result.Text := '';
    try
      for I := 1 to StrToInt(Value) do
      begin
        OneTest;
        if I mod $10000 = 0 then
          Application.ProcessMessages;
      end;
      Result.Text := 'OK';
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

procedure TForm1.Num1Change(Sender: TObject);
begin
  if (Num1.Text <> '') and (Num1.Text <> '$') then
    Uint1.Text := UIntToStr(Value1);
end;

procedure TForm1.Num2Change(Sender: TObject);
begin
  if (Num2.Text <> '') and (Num2.Text <> '$') then
    UInt2.Text := UIntToStr(Value2)
end;

procedure TForm1.UIntCompareButtonClick(Sender: TObject);
begin
  Result.Text := IntToStr(CompareUnsigned(StrToUInt(UInt1.Text), StrToUInt(UInt2.Text)));
end;

procedure TForm1.DivButtonClick(Sender: TObject);
begin
  Result.Text := UIntToStr(UDiv(Value1, Value2));
end;

procedure TForm1.ModButtonClick(Sender: TObject);
begin
  Result.Text := UIntToStr(UMod(Value1, Value2));
end;

end.
{--=>revision-history<=--}
{1 UCOMP.PAS 3-Sep-95,16:40:50,`lisch' CHANGE=INITIAL                         }
{     Initial version.                                                        }
{2 UCOMP.PAS 5-Jan-96,14:52:02,`lisch' CHANGE=FULL_IMPLEMENTATION             }
{     Added conversion to strings, division, remainder.                       }
{3 UCOMP.PAS 5-Jan-96,15:58:34,`lisch' CHANGE=WIN32                           }
{     Completed support for WIN32 (Delphi 2.0), and improved 32-bit support fo}
{--=>revision-history<=--}
